package javax.management.relation;

import java.util.Date;
import javax.management.MBeanServerDelegate;
import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.InvalidAttributeValueException;
import com.sun.jmx.mbeanserver.Util;
import java.util.Set;
import javax.management.InstanceNotFoundException;
import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.ReflectionException;
import javax.management.MBeanException;
import java.util.Iterator;
import com.sun.jmx.defaults.JmxProperties;
import java.util.ArrayList;
import java.util.HashMap;
import javax.management.MBeanServerNotification;
import javax.management.MBeanServer;
import java.util.concurrent.atomic.AtomicLong;
import java.util.List;
import javax.management.ObjectName;
import java.util.Map;
import javax.management.NotificationListener;
import javax.management.MBeanRegistration;
import javax.management.NotificationBroadcasterSupport;

public class RelationService extends NotificationBroadcasterSupport implements RelationServiceMBean, MBeanRegistration, NotificationListener
{
    private Map<String, Object> myRelId2ObjMap;
    private Map<String, String> myRelId2RelTypeMap;
    private Map<ObjectName, String> myRelMBeanObjName2RelIdMap;
    private Map<String, RelationType> myRelType2ObjMap;
    private Map<String, List<String>> myRelType2RelIdsMap;
    private final Map<ObjectName, Map<String, List<String>>> myRefedMBeanObjName2RelIdsMap;
    private boolean myPurgeFlag;
    private final AtomicLong atomicSeqNo;
    private ObjectName myObjName;
    private MBeanServer myMBeanServer;
    private MBeanServerNotificationFilter myUnregNtfFilter;
    private List<MBeanServerNotification> myUnregNtfList;
    
    public RelationService(boolean purgeFlag) {
        this.myRelId2ObjMap = new HashMap();
        this.myRelId2RelTypeMap = new HashMap();
        this.myRelMBeanObjName2RelIdMap = new HashMap();
        this.myRelType2ObjMap = new HashMap();
        this.myRelType2RelIdsMap = new HashMap();
        this.myRefedMBeanObjName2RelIdsMap = new HashMap();
        this.myPurgeFlag = true;
        this.atomicSeqNo = new AtomicLong();
        this.myObjName = null;
        this.myMBeanServer = null;
        this.myUnregNtfFilter = null;
        this.myUnregNtfList = new ArrayList();
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "RelationService");
        this.setPurgeFlag(purgeFlag);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "RelationService");
    }
    
    @Override
    public void isActive() throws RelationServiceNotRegisteredException {
        if (this.myMBeanServer == null) {
            throw new RelationServiceNotRegisteredException("Relation Service not registered in the MBean Server.");
        }
        return;
    }
    
    @Override
    public ObjectName preRegister(MBeanServer myMBeanServer, ObjectName myObjName) throws Exception {
        this.myMBeanServer = myMBeanServer;
        return this.myObjName = myObjName;
    }
    
    @Override
    public void postRegister(Boolean b) {
    }
    
    @Override
    public void preDeregister() throws Exception {
    }
    
    @Override
    public void postDeregister() {
    }
    
    @Override
    public boolean getPurgeFlag() {
        return this.myPurgeFlag;
    }
    
    @Override
    public void setPurgeFlag(boolean myPurgeFlag) {
        this.myPurgeFlag = myPurgeFlag;
    }
    
    @Override
    public void createRelationType(String param1, RoleInfo[] array) throws IllegalArgumentException, InvalidRelationTypeException {
        if (param1 == null || array == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "createRelationType", param1);
        this.addRelationTypeInt(new RelationTypeSupport(param1, array));
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "createRelationType");
        return;
    }
    
    @Override
    public void addRelationType(RelationType relationType) throws IllegalArgumentException, InvalidRelationTypeException {
        if (relationType == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "addRelationType");
        List<RoleInfo> roleInfos = relationType.getRoleInfos();
        if (roleInfos == null) {
            throw new InvalidRelationTypeException("No role info provided.");
        }
        RoleInfo[] array = new RoleInfo[roleInfos.size()];
        int n = 0;
        Iterator<Object> iterator = roleInfos.iterator();
        while (iterator.hasNext()) {
            array[n] = (RoleInfo)iterator.next();
            ++n;
        }
        RelationTypeSupport.checkRoleInfos(array);
        this.addRelationTypeInt(relationType);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "addRelationType");
        return;
    }
    
    @Override
    public List<String> getAllRelationTypeNames() {
        ArrayList list;
        synchronized (this.myRelType2ObjMap) {
            list = new ArrayList(this.myRelType2ObjMap.keySet());
        }
        return list;
    }
    
    @Override
    public List<RoleInfo> getRoleInfos(String param1) throws IllegalArgumentException, RelationTypeNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRoleInfos", param1);
        RelationType relationType = this.getRelationType(param1);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRoleInfos");
        return relationType.getRoleInfos();
    }
    
    @Override
    public RoleInfo getRoleInfo(String s, String s2) throws IllegalArgumentException, RelationTypeNotFoundException, RoleInfoNotFoundException {
        if (s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRoleInfo", new Object[] { s, s2 });
        RoleInfo roleInfo = this.getRelationType(s).getRoleInfo(s2);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRoleInfo");
        return roleInfo;
    }
    
    @Override
    public void removeRelationType(String param1) throws RelationServiceNotRegisteredException, IllegalArgumentException, RelationTypeNotFoundException {
        this.isActive();
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "removeRelationType", param1);
        this.getRelationType(param1);
        ArrayList list = null;
        synchronized (this.myRelType2RelIdsMap) {
            List<Object> c = (List)this.myRelType2RelIdsMap.get(param1);
            if (c != null) {
                list = new ArrayList(c);
            }
        }
        synchronized (this.myRelType2ObjMap) {
            this.myRelType2ObjMap.remove(param1);
        }
        synchronized (this.myRelType2RelIdsMap) {
            this.myRelType2RelIdsMap.remove(param1);
        }
        if (list != null) {
            for (String s : list) {
                try {
                    this.removeRelation(s);
                }
                catch (RelationNotFoundException ex) {
                    throw new RuntimeException(ex.getMessage());
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "removeRelationType");
        return;
    }
    
    @Override
    public void createRelation(String s, String s2, RoleList list) throws RelationServiceNotRegisteredException, IllegalArgumentException, RoleNotFoundException, InvalidRelationIdException, RelationTypeNotFoundException, InvalidRoleValueException {
        this.isActive();
        if (s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "createRelation", new Object[] { s, s2, list });
        this.addRelationInt(true, new RelationSupport(s, this.myObjName, s2, list), null, s, s2, list);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "createRelation");
        return;
    }
    
    @Override
    public void addRelation(ObjectName param1) throws IllegalArgumentException, RelationServiceNotRegisteredException, NoSuchMethodException, InvalidRelationIdException, InstanceNotFoundException, InvalidRelationServiceException, RelationTypeNotFoundException, RoleNotFoundException, InvalidRoleValueException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "addRelation", param1);
        this.isActive();
        if (!this.myMBeanServer.isInstanceOf(param1, "javax.management.relation.Relation")) {
            throw new NoSuchMethodException("This MBean does not implement the Relation interface.");
        }
        String s;
        try {
            s = (String)(String)this.myMBeanServer.getAttribute(param1, "RelationId");
        }
        catch (MBeanException ex) {
            throw new RuntimeException(ex.getTargetException().getMessage());
        }
        catch (ReflectionException ex2) {
            throw new RuntimeException(ex2.getMessage());
        }
        catch (AttributeNotFoundException ex3) {
            throw new RuntimeException(ex3.getMessage());
        }
        if (s == null) {
            throw new InvalidRelationIdException("This MBean does not provide a relation id.");
        }
        ObjectName objectName;
        try {
            objectName = (ObjectName)(ObjectName)this.myMBeanServer.getAttribute(param1, "RelationServiceName");
        }
        catch (MBeanException ex4) {
            throw new RuntimeException(ex4.getTargetException().getMessage());
        }
        catch (ReflectionException ex5) {
            throw new RuntimeException(ex5.getMessage());
        }
        catch (AttributeNotFoundException ex6) {
            throw new RuntimeException(ex6.getMessage());
        }
        boolean n = false;
        if (objectName == null) {
            n = true;
        }
        else if (!objectName.equals(this.myObjName)) {
            n = true;
        }
        if (!(n == 0)) {
            throw new InvalidRelationServiceException("The Relation Service referenced in the MBean is not the current one.");
        }
        String s2;
        try {
            s2 = (String)(String)this.myMBeanServer.getAttribute(param1, "RelationTypeName");
        }
        catch (MBeanException ex7) {
            throw new RuntimeException(ex7.getTargetException().getMessage());
        }
        catch (ReflectionException ex8) {
            throw new RuntimeException(ex8.getMessage());
        }
        catch (AttributeNotFoundException ex9) {
            throw new RuntimeException(ex9.getMessage());
        }
        if (s2 == null) {
            throw new RelationTypeNotFoundException("No relation type provided.");
        }
        RoleList list;
        try {
            list = (RoleList)(RoleList)this.myMBeanServer.invoke(param1, "retrieveAllRoles", null, (String[])null);
        }
        catch (MBeanException ex10) {
            throw new RuntimeException(ex10.getTargetException().getMessage());
        }
        catch (ReflectionException ex11) {
            throw new RuntimeException(ex11.getMessage());
        }
        this.addRelationInt(false, null, param1, s, s2, list);
        synchronized (this.myRelMBeanObjName2RelIdMap) {
            this.myRelMBeanObjName2RelIdMap.put(param1, s);
        }
        try {
            this.myMBeanServer.setAttribute(param1, new Attribute("RelationServiceManagementFlag", Boolean.TRUE));
        }
        catch (Exception ex12) {}
        ArrayList o = new ArrayList();
        o.add(param1);
        this.updateUnregistrationListener(o, null);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "addRelation");
        return;
    }
    
    @Override
    public ObjectName isRelationMBean(String param1) throws IllegalArgumentException, RelationNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "isRelationMBean", param1);
        Object relation = this.getRelation(param1);
        if ((relation instanceof ObjectName)) {
            return (ObjectName)relation;
        }
        return null;
    }
    
    @Override
    public String isRelation(ObjectName param1) throws IllegalArgumentException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "isRelation", param1);
        String s = null;
        synchronized (this.myRelMBeanObjName2RelIdMap) {
            String s2 = (String)this.myRelMBeanObjName2RelIdMap.get(param1);
            if (s2 != null) {
                s = s2;
            }
        }
        return s;
    }
    
    @Override
    public Boolean hasRelation(String param1) throws IllegalArgumentException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "hasRelation", param1);
        try {
            this.getRelation(param1);
            return Boolean.valueOf(true);
        }
        catch (RelationNotFoundException ex) {
            return Boolean.valueOf(false);
        }
    }
    
    @Override
    public List<String> getAllRelationIds() {
        ArrayList list;
        synchronized (this.myRelId2ObjMap) {
            list = new ArrayList(this.myRelId2ObjMap.keySet());
        }
        return list;
    }
    
    @Override
    public Integer checkRoleReading(String s, String s2) throws IllegalArgumentException, RelationTypeNotFoundException {
        if (s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "checkRoleReading", new Object[] { s, s2 });
        RelationType relationType = this.getRelationType(s2);
        Integer n;
        try {
            n = this.checkRoleInt(1, s, null, relationType.getRoleInfo(s), false);
        }
        catch (RoleInfoNotFoundException ex) {
            n = Integer.valueOf(1);
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleReading");
        return n;
    }
    
    @Override
    public Integer checkRoleWriting(Role role, String s, Boolean b) throws IllegalArgumentException, RelationTypeNotFoundException {
        if (role == null || s == null || b == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "checkRoleWriting", new Object[] { role, s, b });
        RelationType relationType = this.getRelationType(s);
        String roleName = role.getRoleName();
        List<ObjectName> roleValue = role.getRoleValue();
        boolean n = true;
        if ((boolean)b) {
            n = false;
        }
        RoleInfo roleInfo;
        try {
            roleInfo = relationType.getRoleInfo(roleName);
        }
        catch (RoleInfoNotFoundException ex) {
            JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleWriting");
            return Integer.valueOf(1);
        }
        Integer checkRoleInt = this.checkRoleInt(2, roleName, roleValue, roleInfo, (boolean)n);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleWriting");
        return checkRoleInt;
    }
    
    @Override
    public void sendRelationCreationNotification(String s) throws IllegalArgumentException, RelationNotFoundException {
        if (s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "sendRelationCreationNotification", s);
        StringBuilder sb = new StringBuilder("Creation of relation ");
        sb.append(s);
        this.sendNotificationInt(1, sb.toString(), s, null, null, null, null);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "sendRelationCreationNotification");
        return;
    }
    
    @Override
    public void sendRoleUpdateNotification(String s, Role role, List<ObjectName> c) throws IllegalArgumentException, RelationNotFoundException {
        if (s == null || role == null || c == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        if (!(c instanceof ArrayList)) {
            c = new ArrayList(c);
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "sendRoleUpdateNotification", new Object[] { s, role, c });
        String roleName = role.getRoleName();
        List<ObjectName> roleValue = role.getRoleValue();
        String roleValueToString = Role.roleValueToString(roleValue);
        String roleValueToString2 = Role.roleValueToString(c);
        StringBuilder o = new StringBuilder("Value of role ");
        o.append(roleName);
        o.append(" has changed\nOld value:\n");
        o.append(roleValueToString2);
        o.append("\nNew value:\n");
        o.append(roleValueToString);
        this.sendNotificationInt(2, o.toString(), s, null, roleName, roleValue, c);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "sendRoleUpdateNotification");
        return;
    }
    
    @Override
    public void sendRelationRemovalNotification(String str, List<ObjectName> list) throws IllegalArgumentException, RelationNotFoundException {
        if (str == null) {
            throw new IllegalArgumentException("Invalid parameter");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "sendRelationRemovalNotification", new Object[] { str, list });
        this.sendNotificationInt(3, "Removal of relation " + str, str, list, null, null, null);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "sendRelationRemovalNotification");
        return;
    }
    
    @Override
    public void updateRoleMap(String s, Role role, List<ObjectName> c) throws IllegalArgumentException, RelationServiceNotRegisteredException, RelationNotFoundException {
        if (s == null || role == null || c == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "updateRoleMap", new Object[] { s, role, c });
        this.isActive();
        this.getRelation(s);
        String roleName = role.getRoleName();
        List<ObjectName> roleValue = role.getRoleValue();
        ArrayList list = new ArrayList(c);
        ArrayList<ObjectName> list2 = new ArrayList();
        for (ObjectName objectName : roleValue) {
            int index = list.indexOf(objectName);
            if (index == -1) {
                if (this.addNewMBeanReference(objectName, s, roleName)) {
                    list2.add(objectName);
                }
                else {
                    continue;
                }
            }
            else {
                list.remove(index);
            }
        }
        ArrayList o = new ArrayList();
        for (ObjectName objectName2 : list) {
            if (this.removeMBeanReference(objectName2, s, roleName, false)) {
                o.add(objectName2);
            }
            else {
                continue;
            }
        }
        this.updateUnregistrationListener(list2, o);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "updateRoleMap");
        return;
    }
    
    @Override
    public void removeRelation(String param1) throws RelationServiceNotRegisteredException, IllegalArgumentException, RelationNotFoundException {
        this.isActive();
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "removeRelation", param1);
        Object relation = this.getRelation(param1);
        if ((relation instanceof ObjectName)) {
            ArrayList<ObjectName> list = new ArrayList();
            list.add((ObjectName)relation);
            this.updateUnregistrationListener(null, list);
        }
        this.sendRelationRemovalNotification(param1, null);
        ArrayList o = new ArrayList();
        ArrayList o2 = new ArrayList();
        synchronized (this.myRefedMBeanObjName2RelIdsMap) {
            for (ObjectName objectName : this.myRefedMBeanObjName2RelIdsMap.keySet()) {
                Map<Object, Object> map = (Map)this.myRefedMBeanObjName2RelIdsMap.get(objectName);
                if (map.containsKey(param1)) {
                    map.remove(param1);
                    o.add(objectName);
                }
                if (map.isEmpty()) {
                    o2.add(objectName);
                }
                else {
                    continue;
                }
            }
            Iterator<Object> iterator2 = o2.iterator();
            while (iterator2.hasNext()) {
                this.myRefedMBeanObjName2RelIdsMap.remove((Object)iterator2.next());
            }
        }
        synchronized (this.myRelId2ObjMap) {
            this.myRelId2ObjMap.remove(param1);
        }
        if ((relation instanceof ObjectName)) {
            synchronized (this.myRelMBeanObjName2RelIdMap) {
                this.myRelMBeanObjName2RelIdMap.remove((ObjectName)relation);
            }
        }
        Map<Object, Object> map2;
        synchronized (this.myRelId2RelTypeMap) {
            map2 = (String)this.myRelId2RelTypeMap.get(param1);
            this.myRelId2RelTypeMap.remove(param1);
        }
        synchronized (this.myRelType2RelIdsMap) {
            List<Object> list2 = (List)this.myRelType2RelIdsMap.get(map2);
            if (list2 != null) {
                list2.remove(param1);
                if (list2.isEmpty()) {
                    this.myRelType2RelIdsMap.remove(map2);
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "removeRelation");
        return;
    }
    
    @Override
    public void purgeRelations() throws RelationServiceNotRegisteredException {
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "purgeRelations");
        this.isActive();
        ArrayList list;
        synchronized (this.myRefedMBeanObjName2RelIdsMap) {
            list = new ArrayList(this.myUnregNtfList);
            this.myUnregNtfList = new ArrayList();
        }
        ArrayList<ObjectName> list2 = new ArrayList();
        HashMap<ObjectName, Map<String, List>> hashMap = new HashMap();
        synchronized (this.myRefedMBeanObjName2RelIdsMap) {
            Iterator<Object> iterator = list.iterator();
            while (iterator.hasNext()) {
                ObjectName mBeanName = ((MBeanServerNotification)iterator.next()).getMBeanName();
                list2.add(mBeanName);
                hashMap.put(mBeanName, (Map<String, List<String>>)this.myRefedMBeanObjName2RelIdsMap.get(mBeanName));
                this.myRefedMBeanObjName2RelIdsMap.remove(mBeanName);
            }
        }
        this.updateUnregistrationListener(null, list2);
        Iterator<Object> iterator2 = list.iterator();
        while (iterator2.hasNext()) {
            ObjectName mBeanName2 = ((MBeanServerNotification)iterator2.next()).getMBeanName();
            for (Map.Entry entry : ((Map)hashMap.get(mBeanName2)).entrySet()) {
                String s = (String)entry.getKey();
                List<String> list3 = (List)entry.getValue();
                try {
                    this.handleReferenceUnregistration(s, mBeanName2, list3);
                }
                catch (RelationNotFoundException ex) {
                    throw new RuntimeException(ex.getMessage());
                }
                catch (RoleNotFoundException ex2) {
                    throw new RuntimeException(ex2.getMessage());
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "purgeRelations");
    }
    
    @Override
    public Map<String, List<String>> findReferencingRelations(ObjectName objectName, String anObject, String s) throws IllegalArgumentException {
        if (objectName == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "findReferencingRelations", new Object[] { objectName, anObject, s });
        HashMap hashMap = new HashMap();
        synchronized (this.myRefedMBeanObjName2RelIdsMap) {
            Map<Object, Object> map = (Map)this.myRefedMBeanObjName2RelIdsMap.get(objectName);
            if (map != null) {
                Set<Object> keySet = map.keySet();
                ArrayList list;
                if (anObject == null) {
                    list = new ArrayList(keySet);
                }
                else {
                    list = new ArrayList();
                    for (String s2 : keySet) {
                        String s3;
                        synchronized (this.myRelId2RelTypeMap) {
                            s3 = (String)this.myRelId2RelTypeMap.get(s2);
                        }
                        if (s3.equals(anObject)) {
                            list.add(s2);
                        }
                        else {
                            continue;
                        }
                    }
                }
                for (String s4 : list) {
                    List<Object> c = (List)map.get(s4);
                    if (s == null) {
                        hashMap.put(s4, new ArrayList(c));
                    }
                    else if (c.contains(s)) {
                        ArrayList o = new ArrayList();
                        o.add(s);
                        hashMap.put(s4, o);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "findReferencingRelations");
        return hashMap;
    }
    
    @Override
    public Map<ObjectName, List<String>> findAssociatedMBeans(ObjectName object, String s, String s2) throws IllegalArgumentException {
        if (object == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "findAssociatedMBeans", new Object[] { object, s, s2 });
        Map<String, List<String>> referencingRelations = this.findReferencingRelations(object, s, s2);
        HashMap<ObjectName, ArrayList> hashMap = new HashMap();
        for (String s3 : referencingRelations.keySet()) {
            Map<ObjectName, List<String>> referencedMBeans;
            try {
                referencedMBeans = this.getReferencedMBeans(s3);
            }
            catch (RelationNotFoundException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            for (ObjectName objectName : referencedMBeans.keySet()) {
                if (!objectName.equals(object)) {
                    List<String> list = (List)hashMap.get(objectName);
                    if (list == null) {
                        ArrayList<String> list2 = new ArrayList();
                        list2.add(s3);
                        hashMap.put(objectName, list2);
                    }
                    else {
                        list.add(s3);
                    }
                }
                else {
                    continue;
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "findAssociatedMBeans");
        return hashMap;
    }
    
    @Override
    public List<String> findRelationsOfType(String s) throws IllegalArgumentException, RelationTypeNotFoundException {
        if (s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "findRelationsOfType");
        this.getRelationType(s);
        ArrayList<Object> list;
        synchronized (this.myRelType2RelIdsMap) {
            List<Object> c = (List)this.myRelType2RelIdsMap.get(s);
            if (c == null) {
                list = new ArrayList();
            }
            else {
                list = new ArrayList(c);
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "findRelationsOfType");
        return list;
    }
    
    @Override
    public List<ObjectName> getRole(String s, String s2) throws RelationServiceNotRegisteredException, IllegalArgumentException, RelationNotFoundException, RoleNotFoundException {
        if (s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRole", new Object[] { s, s2 });
        this.isActive();
        Object relation = this.getRelation(s);
        List<Object> list;
        if ((relation instanceof RelationSupport)) {
            list = (List)Util.cast(((RelationSupport)relation).getRoleInt(s2, true, this, false));
        }
        else {
            Object[] array = { s2 };
            String[] array2 = { "java.lang.String" };
            try {
                List<Object> c = (List)Util.cast(this.myMBeanServer.invoke((ObjectName)relation, "getRole", array, array2));
                if (c == null || c instanceof ArrayList) {
                    list = c;
                }
                else {
                    list = new ArrayList(c);
                }
            }
            catch (InstanceNotFoundException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (ReflectionException ex2) {
                throw new RuntimeException(ex2.getMessage());
            }
            catch (MBeanException ex3) {
                Exception targetException = ex3.getTargetException();
                if ((targetException instanceof RoleNotFoundException)) {
                    throw (RoleNotFoundException)targetException;
                }
                throw new RuntimeException(targetException.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRole");
        return list;
    }
    
    @Override
    public RoleResult getRoles(String param1, String[] array) throws RelationServiceNotRegisteredException, IllegalArgumentException, RelationNotFoundException {
        if (param1 == null || array == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRoles", param1);
        this.isActive();
        Object relation = this.getRelation(param1);
        RoleResult rolesInt;
        if ((relation instanceof RelationSupport)) {
            rolesInt = ((RelationSupport)relation).getRolesInt(array, true, this);
        }
        else {
            Object[] array2 = { array };
            String[] array3 = { null };
            try {
                array3[0] = array.getClass().getName();
            }
            catch (Exception ex) {}
            try {
                rolesInt = (RoleResult)(RoleResult)this.myMBeanServer.invoke((ObjectName)relation, "getRoles", array2, array3);
            }
            catch (InstanceNotFoundException ex2) {
                throw new RuntimeException(ex2.getMessage());
            }
            catch (ReflectionException ex3) {
                throw new RuntimeException(ex3.getMessage());
            }
            catch (MBeanException ex4) {
                throw new RuntimeException(ex4.getTargetException().getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRoles");
        return rolesInt;
    }
    
    @Override
    public RoleResult getAllRoles(String param1) throws IllegalArgumentException, RelationNotFoundException, RelationServiceNotRegisteredException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRoles", param1);
        Object relation = this.getRelation(param1);
        RoleResult allRolesInt;
        if ((relation instanceof RelationSupport)) {
            allRolesInt = ((RelationSupport)relation).getAllRolesInt(true, this);
        }
        else {
            try {
                allRolesInt = (RoleResult)(RoleResult)this.myMBeanServer.getAttribute((ObjectName)relation, "AllRoles");
            }
            catch (Exception ex) {
                throw new RuntimeException(ex.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRoles");
        return allRolesInt;
    }
    
    @Override
    public Integer getRoleCardinality(String s, String s2) throws IllegalArgumentException, RelationNotFoundException, RoleNotFoundException {
        if (s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRoleCardinality", new Object[] { s, s2 });
        Object relation = this.getRelation(s);
        Integer roleCardinality;
        if ((relation instanceof RelationSupport)) {
            roleCardinality = ((RelationSupport)relation).getRoleCardinality(s2);
        }
        else {
            Object[] array = { s2 };
            String[] array2 = { "java.lang.String" };
            try {
                roleCardinality = (Integer)(Integer)this.myMBeanServer.invoke((ObjectName)relation, "getRoleCardinality", array, array2);
            }
            catch (InstanceNotFoundException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (ReflectionException ex2) {
                throw new RuntimeException(ex2.getMessage());
            }
            catch (MBeanException ex3) {
                Exception targetException = ex3.getTargetException();
                if ((targetException instanceof RoleNotFoundException)) {
                    throw (RoleNotFoundException)targetException;
                }
                throw new RuntimeException(targetException.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRoleCardinality");
        return roleCardinality;
    }
    
    @Override
    public void setRole(String s, Role value) throws RelationServiceNotRegisteredException, IllegalArgumentException, RelationNotFoundException, RoleNotFoundException, InvalidRoleValueException {
        if (s == null || value == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "setRole", new Object[] { s, value });
        this.isActive();
        Object relation = this.getRelation(s);
        if ((relation instanceof RelationSupport)) {
            try {
                ((RelationSupport)relation).setRoleInt(value, true, this, false);
            }
            catch (RelationTypeNotFoundException ex6) {
                throw new RuntimeException(ex6.getMessage());
            }
        }
        else {
            (new Object[1])[0] = value;
            (new String[1])[0] = "javax.management.relation.Role";
            try {
                this.myMBeanServer.setAttribute((ObjectName)relation, new Attribute("Role", value));
            }
            catch (InstanceNotFoundException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (ReflectionException ex2) {
                throw new RuntimeException(ex2.getMessage());
            }
            catch (MBeanException ex3) {
                Exception targetException = ex3.getTargetException();
                if ((targetException instanceof RoleNotFoundException)) {
                    throw (RoleNotFoundException)targetException;
                }
                if ((targetException instanceof InvalidRoleValueException)) {
                    throw (InvalidRoleValueException)targetException;
                }
                throw new RuntimeException(targetException.getMessage());
            }
            catch (AttributeNotFoundException ex4) {
                throw new RuntimeException(ex4.getMessage());
            }
            catch (InvalidAttributeValueException ex5) {
                throw new RuntimeException(ex5.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "setRole");
        return;
    }
    
    @Override
    public RoleResult setRoles(String s, RoleList list) throws RelationServiceNotRegisteredException, IllegalArgumentException, RelationNotFoundException {
        if (s == null || list == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "setRoles", new Object[] { s, list });
        this.isActive();
        Object relation = this.getRelation(s);
        RoleResult setRolesInt;
        if ((relation instanceof RelationSupport)) {
            try {
                setRolesInt = ((RelationSupport)relation).setRolesInt(list, true, this);
            }
            catch (RelationTypeNotFoundException ex4) {
                throw new RuntimeException(ex4.getMessage());
            }
        }
        else {
            Object[] array = { list };
            String[] array2 = { "javax.management.relation.RoleList" };
            try {
                setRolesInt = (RoleResult)(RoleResult)this.myMBeanServer.invoke((ObjectName)relation, "setRoles", array, array2);
            }
            catch (InstanceNotFoundException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (ReflectionException ex2) {
                throw new RuntimeException(ex2.getMessage());
            }
            catch (MBeanException ex3) {
                throw new RuntimeException(ex3.getTargetException().getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "setRoles");
        return setRolesInt;
    }
    
    @Override
    public Map<ObjectName, List<String>> getReferencedMBeans(String param1) throws IllegalArgumentException, RelationNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getReferencedMBeans", param1);
        Object relation = this.getRelation(param1);
        Map<ObjectName, List<String>> referencedMBeans;
        if ((relation instanceof RelationSupport)) {
            referencedMBeans = ((RelationSupport)relation).getReferencedMBeans();
        }
        else {
            try {
                referencedMBeans = (Map)Util.cast(this.myMBeanServer.getAttribute((ObjectName)relation, "ReferencedMBeans"));
            }
            catch (Exception ex) {
                throw new RuntimeException(ex.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getReferencedMBeans");
        return referencedMBeans;
    }
    
    @Override
    public String getRelationTypeName(String param1) throws IllegalArgumentException, RelationNotFoundException {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRelationTypeName", param1);
        Object relation = this.getRelation(param1);
        String relationTypeName;
        if ((relation instanceof RelationSupport)) {
            relationTypeName = ((RelationSupport)relation).getRelationTypeName();
        }
        else {
            try {
                relationTypeName = (String)(String)this.myMBeanServer.getAttribute((ObjectName)relation, "RelationTypeName");
            }
            catch (Exception ex) {
                throw new RuntimeException(ex.getMessage());
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRelationTypeName");
        return relationTypeName;
    }
    
    @Override
    public void handleNotification(Notification param1, Object o) {
        if (param1 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "handleNotification", param1);
        if ((param1 instanceof MBeanServerNotification)) {
            MBeanServerNotification mBeanServerNotification = (MBeanServerNotification)param1;
            if (param1.getType().equals("JMX.mbean.unregistered")) {
                ObjectName mBeanName = ((MBeanServerNotification)param1).getMBeanName();
                boolean n = false;
                synchronized (this.myRefedMBeanObjName2RelIdsMap) {
                    if (this.myRefedMBeanObjName2RelIdsMap.containsKey(mBeanName)) {
                        synchronized (this.myUnregNtfList) {
                            this.myUnregNtfList.add(mBeanServerNotification);
                        }
                        n = true;
                    }
                    if (n && this.myPurgeFlag) {
                        try {
                            this.purgeRelations();
                        }
                        catch (Exception ex) {
                            throw new RuntimeException(ex.getMessage());
                        }
                    }
                }
                Map<Object, Object> map;
                synchronized (this.myRelMBeanObjName2RelIdMap) {
                    map = (String)this.myRelMBeanObjName2RelIdMap.get(mBeanName);
                }
                if (map != null) {
                    try {
                        this.removeRelation(map);
                    }
                    catch (Exception ex2) {
                        throw new RuntimeException(ex2.getMessage());
                    }
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "handleNotification");
        return;
    }
    
    @Override
    public MBeanNotificationInfo[] getNotificationInfo() {
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getNotificationInfo");
        MBeanNotificationInfo mBeanNotificationInfo = new MBeanNotificationInfo(new String[] { "jmx.relation.creation.basic", "jmx.relation.creation.mbean", "jmx.relation.update.basic", "jmx.relation.update.mbean", "jmx.relation.removal.basic", "jmx.relation.removal.mbean" }, "javax.management.relation.RelationNotification", "Sent when a relation is created, updated or deleted.");
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getNotificationInfo");
        return new MBeanNotificationInfo[] { mBeanNotificationInfo };
    }
    
    private void addRelationTypeInt(RelationType relationType) throws IllegalArgumentException, InvalidRelationTypeException {
        if (relationType == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "addRelationTypeInt");
        String relationTypeName = relationType.getRelationTypeName();
        try {
            if (this.getRelationType(relationTypeName) != null) {
                StringBuilder sb = new StringBuilder("There is already a relation type in the Relation Service with name ");
                sb.append(relationTypeName);
                throw new InvalidRelationTypeException(sb.toString());
            }
        }
        catch (RelationTypeNotFoundException ex) {}
        synchronized (this.myRelType2ObjMap) {
            this.myRelType2ObjMap.put(relationTypeName, relationType);
        }
        if ((relationType instanceof RelationTypeSupport)) {
            ((RelationTypeSupport)relationType).setRelationServiceFlag(true);
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "addRelationTypeInt");
        return;
    }
    
    RelationType getRelationType(String s) throws IllegalArgumentException, RelationTypeNotFoundException {
        if (s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRelationType", s);
        RelationType relationType;
        synchronized (this.myRelType2ObjMap) {
            relationType = (RelationType)this.myRelType2ObjMap.get(s);
        }
        if (relationType == null) {
            StringBuilder sb = new StringBuilder("No relation type created in the Relation Service with the name ");
            sb.append(s);
            throw new RelationTypeNotFoundException(sb.toString());
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRelationType");
        return relationType;
    }
    
    Object getRelation(String s) throws IllegalArgumentException, RelationNotFoundException {
        if (s == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "getRelation", s);
        Object value;
        synchronized (this.myRelId2ObjMap) {
            value = this.myRelId2ObjMap.get(s);
        }
        if (value == null) {
            throw new RelationNotFoundException("No relation associated to relation id " + s);
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "getRelation");
        return value;
    }
    
    private boolean addNewMBeanReference(ObjectName objectName, String s, String s2) throws IllegalArgumentException {
        if (objectName == null || s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "addNewMBeanReference", new Object[] { objectName, s, s2 });
        boolean n = false;
        synchronized (this.myRefedMBeanObjName2RelIdsMap) {
            Map<String, ArrayList> map = (Map)this.myRefedMBeanObjName2RelIdsMap.get(objectName);
            if (map == null) {
                n = true;
                ArrayList<String> list3 = new ArrayList();
                list3.add(s2);
                HashMap<String, ArrayList> hashMap = new HashMap();
                hashMap.put(s, list3);
                this.myRefedMBeanObjName2RelIdsMap.put(objectName, hashMap);
            }
            else {
                List<String> list = (List)map.get(s);
                if (list == null) {
                    ArrayList<String> list2 = new ArrayList();
                    list2.add(s2);
                    map.put(s, list2);
                }
                else {
                    list.add(s2);
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "addNewMBeanReference");
        return n;
    }
    
    private boolean removeMBeanReference(ObjectName objectName, String s, String s2, boolean b) throws IllegalArgumentException {
        if (objectName == null || s == null || s2 == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "removeMBeanReference", new Object[] { objectName, s, s2, Boolean.valueOf(b) });
        boolean n = false;
        synchronized (this.myRefedMBeanObjName2RelIdsMap) {
            Map<Object, Object> map = (Map)this.myRefedMBeanObjName2RelIdsMap.get(objectName);
            if (map == null) {
                JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "removeMBeanReference");
                return true;
            }
            List<Object> list = null;
            if (!b) {
                list = (List)map.get(s);
                int index = list.indexOf(s2);
                if (index != -1) {
                    list.remove(index);
                }
            }
            if (list.isEmpty() || b) {
                map.remove(s);
            }
            if (map.isEmpty()) {
                this.myRefedMBeanObjName2RelIdsMap.remove(objectName);
                n = true;
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "removeMBeanReference");
        return n;
    }
    
    private void updateUnregistrationListener(List<ObjectName> list, List<ObjectName> list2) throws RelationServiceNotRegisteredException {
        if (list != null && list2 != null && list.isEmpty() && list2.isEmpty()) {
            return;
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "updateUnregistrationListener", new Object[] { list, list2 });
        this.isActive();
        if (list != null || list2 != null) {
            boolean n = false;
            if (this.myUnregNtfFilter == null) {
                this.myUnregNtfFilter = new MBeanServerNotificationFilter();
                n = true;
            }
            synchronized (this.myUnregNtfFilter) {
                if (list != null) {
                    Iterator<Object> iterator = list.iterator();
                    while (iterator.hasNext()) {
                        this.myUnregNtfFilter.enableObjectName((ObjectName)iterator.next());
                    }
                }
                if (list2 != null) {
                    Iterator<Object> iterator2 = list2.iterator();
                    while (iterator2.hasNext()) {
                        this.myUnregNtfFilter.disableObjectName((ObjectName)iterator2.next());
                    }
                }
                if (!(n == 0)) {
                    try {
                        this.myMBeanServer.addNotificationListener(MBeanServerDelegate.DELEGATE_NAME, this, this.myUnregNtfFilter, null);
                    }
                    catch (InstanceNotFoundException ex) {
                        throw new RelationServiceNotRegisteredException(ex.getMessage());
                    }
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "updateUnregistrationListener");
        return;
    }
    
    private void addRelationInt(boolean b, RelationSupport relationSupport, ObjectName objectName, String str, String s, RoleList list) throws IllegalArgumentException, RelationServiceNotRegisteredException, RoleNotFoundException, InvalidRelationIdException, RelationTypeNotFoundException, InvalidRoleValueException {
        if (str == null || s == null || (b && (relationSupport == null || objectName != null)) || (!b && (objectName == null || relationSupport != null))) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "addRelationInt", new Object[] { Boolean.valueOf(b), relationSupport, objectName, str, s, list });
        this.isActive();
        try {
            if (this.getRelation(str) != null) {
                StringBuilder sb = new StringBuilder("There is already a relation with id ");
                sb.append(str);
                throw new InvalidRelationIdException(sb.toString());
            }
        }
        catch (RelationNotFoundException ex) {}
        RelationType relationType = this.getRelationType(s);
        ArrayList o = new ArrayList(relationType.getRoleInfos());
        if (list != null) {
            for (Role role : list.asList()) {
                String roleName = role.getRoleName();
                List<ObjectName> roleValue = role.getRoleValue();
                RoleInfo roleInfo;
                try {
                    roleInfo = relationType.getRoleInfo(roleName);
                }
                catch (RoleInfoNotFoundException ex2) {
                    throw new RoleNotFoundException(ex2.getMessage());
                }
                int intValue = (int)this.checkRoleInt(2, roleName, roleValue, roleInfo, false);
                if (intValue != 0) {
                    throwRoleProblemException(intValue, roleName);
                }
                o.remove(o.indexOf(roleInfo));
            }
        }
        this.initializeMissingRoles(b, relationSupport, objectName, str, s, o);
        synchronized (this.myRelId2ObjMap) {
            if (b) {
                this.myRelId2ObjMap.put(str, relationSupport);
            }
            else {
                this.myRelId2ObjMap.put(str, objectName);
            }
        }
        synchronized (this.myRelId2RelTypeMap) {
            this.myRelId2RelTypeMap.put(str, s);
        }
        synchronized (this.myRelType2RelIdsMap) {
            List<String> list2 = (List<String>)this.myRelType2RelIdsMap.get(s);
            boolean n = false;
            if (list2 == null) {
                n = true;
                list2 = new ArrayList();
            }
            list2.add(str);
            if (!(n == 0)) {
                this.myRelType2RelIdsMap.put(s, list2);
            }
        }
        for (Role role2 : list.asList()) {
            ArrayList o2 = new ArrayList();
            try {
                this.updateRoleMap(str, role2, o2);
            }
            catch (RelationNotFoundException ex3) {}
        }
        try {
            this.sendRelationCreationNotification(str);
        }
        catch (RelationNotFoundException ex4) {}
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "addRelationInt");
        return;
    }
    
    private Integer checkRoleInt(int i, String s, List<ObjectName> list, RoleInfo roleInfo, boolean b) throws IllegalArgumentException {
        if (s == null || roleInfo == null || (i == 2 && list == null)) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "checkRoleInt", new Object[] { Integer.valueOf(i), s, list, roleInfo, Boolean.valueOf(b) });
        if (!s.equals(roleInfo.getName())) {
            JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
            return Integer.valueOf(1);
        }
        if (i == 1) {
            if (!roleInfo.isReadable()) {
                JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                return Integer.valueOf(2);
            }
            JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
            return new Integer(0);
        }
        else {
            if (b && !roleInfo.isWritable()) {
                JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                return new Integer(3);
            }
            int size = list.size();
            if (!roleInfo.checkMinDegree(size)) {
                JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                return new Integer(4);
            }
            if (!roleInfo.checkMaxDegree(size)) {
                JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                return new Integer(5);
            }
            String refMBeanClassName = roleInfo.getRefMBeanClassName();
            for (ObjectName objectName : list) {
                if (objectName == null) {
                    JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                    return new Integer(7);
                }
                try {
                    if (!this.myMBeanServer.isInstanceOf(objectName, refMBeanClassName)) {
                        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                        return new Integer(6);
                    }
                    continue;
                }
                catch (InstanceNotFoundException ex) {
                    JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
                    return new Integer(7);
                }
            }
            JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "checkRoleInt");
            return new Integer(0);
        }
    }
    
    private void initializeMissingRoles(boolean b, RelationSupport relationSupport, ObjectName objectName, String s, String s2, List<RoleInfo> list) throws IllegalArgumentException, RelationServiceNotRegisteredException, InvalidRoleValueException {
        if ((b && (relationSupport == null || objectName != null)) || (!b && (objectName == null || relationSupport != null)) || s == null || s2 == null || list == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "initializeMissingRoles", new Object[] { Boolean.valueOf(b), relationSupport, objectName, s, s2, list });
        this.isActive();
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            Role value = new Role(((RoleInfo)iterator.next()).getName(), new ArrayList());
            if (b) {
                try {
                    relationSupport.setRoleInt(value, true, this, false);
                }
                catch (RoleNotFoundException ex6) {
                    throw new RuntimeException(ex6.getMessage());
                }
                catch (RelationNotFoundException ex7) {
                    throw new RuntimeException(ex7.getMessage());
                }
                catch (RelationTypeNotFoundException ex8) {
                    throw new RuntimeException(ex8.getMessage());
                }
            }
            else {
                (new Object[1])[0] = value;
                (new String[1])[0] = "javax.management.relation.Role";
                try {
                    this.myMBeanServer.setAttribute(objectName, new Attribute("Role", value));
                }
                catch (InstanceNotFoundException ex) {
                    throw new RuntimeException(ex.getMessage());
                }
                catch (ReflectionException ex2) {
                    throw new RuntimeException(ex2.getMessage());
                }
                catch (MBeanException ex3) {
                    Exception targetException = ex3.getTargetException();
                    if ((targetException instanceof InvalidRoleValueException)) {
                        throw (InvalidRoleValueException)targetException;
                    }
                    throw new RuntimeException(targetException.getMessage());
                }
                catch (AttributeNotFoundException ex4) {
                    throw new RuntimeException(ex4.getMessage());
                }
                catch (InvalidAttributeValueException ex5) {
                    throw new RuntimeException(ex5.getMessage());
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "initializeMissingRoles");
        return;
    }
    
    static void throwRoleProblemException(int n, String str) throws IllegalArgumentException, RoleNotFoundException, InvalidRoleValueException {
        if (str == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        int n2 = 0;
        String str2 = null;
        switch (n) {
            case 1: {
                str2 = " does not exist in relation.";
                n2 = 1;
                break;
            }
            case 2: {
                str2 = " is not readable.";
                n2 = 1;
                break;
            }
            case 3: {
                str2 = " is not writable.";
                n2 = 1;
                break;
            }
            case 4: {
                str2 = " has a number of MBean references less than the expected minimum degree.";
                n2 = 2;
                break;
            }
            case 5: {
                str2 = " has a number of MBean references greater than the expected maximum degree.";
                n2 = 2;
                break;
            }
            case 6: {
                str2 = " has an MBean reference to an MBean not of the expected class of references for that role.";
                n2 = 2;
                break;
            }
            case 7: {
                str2 = " has a reference to null or to an MBean not registered.";
                n2 = 2;
                break;
            }
        }
        StringBuilder o = new StringBuilder(str);
        o.append(str2);
        String string = o.toString();
        if (n2 == 1) {
            throw new RoleNotFoundException(string);
        }
        if (n2 == 2) {
            throw new InvalidRoleValueException(string);
        }
        return;
    }
    
    private void sendNotificationInt(int i, String s, String s2, List<ObjectName> list, String s3, List<ObjectName> list2, List<ObjectName> list3) throws IllegalArgumentException, RelationNotFoundException {
        if (s == null || s2 == null || (i != 3 && list != null) || (i == 2 && (s3 == null || list2 == null || list3 == null))) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "sendNotificationInt", new Object[] { Integer.valueOf(i), s, s2, list, s3, list2, list3 });
        String s4;
        synchronized (this.myRelId2RelTypeMap) {
            s4 = (String)this.myRelId2RelTypeMap.get(s2);
        }
        ObjectName relationMBean = this.isRelationMBean(s2);
        String s5 = null;
        if (relationMBean != null) {
            switch (i) {
                case 1: {
                    s5 = "jmx.relation.creation.mbean";
                    break;
                }
                case 2: {
                    s5 = "jmx.relation.update.mbean";
                    break;
                }
                case 3: {
                    s5 = "jmx.relation.removal.mbean";
                    break;
                }
            }
        }
        else {
            switch (i) {
                case 1: {
                    s5 = "jmx.relation.creation.basic";
                    break;
                }
                case 2: {
                    s5 = "jmx.relation.update.basic";
                    break;
                }
                case 3: {
                    s5 = "jmx.relation.removal.basic";
                    break;
                }
            }
        }
        Long value = Long.valueOf(this.atomicSeqNo.incrementAndGet());
        long time = new Date().getTime();
        Object notification = null;
        if (s5.equals("jmx.relation.creation.basic") || s5.equals("jmx.relation.creation.mbean") || s5.equals("jmx.relation.removal.basic") || s5.equals("jmx.relation.removal.mbean")) {
            notification = new RelationNotification(s5, this, (long)value, time, s, s2, s4, relationMBean, list);
        }
        else if (s5.equals("jmx.relation.update.basic") || s5.equals("jmx.relation.update.mbean")) {
            notification = new RelationNotification(s5, this, (long)value, time, s, s2, s4, relationMBean, s3, list2, list3);
        }
        this.sendNotification(notification);
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "sendNotificationInt");
        return;
    }
    
    private void handleReferenceUnregistration(String s, ObjectName objectName, List<String> list) throws IllegalArgumentException, RelationServiceNotRegisteredException, RelationNotFoundException, RoleNotFoundException {
        if (s == null || list == null || objectName == null) {
            throw new IllegalArgumentException("Invalid parameter.");
        }
        JmxProperties.RELATION_LOGGER.entering(RelationService.class.getName(), "handleReferenceUnregistration", new Object[] { s, objectName, list });
        this.isActive();
        String relationTypeName = this.getRelationTypeName(s);
        Object relation = this.getRelation(s);
        int n = 0;
        for (String s2 : list) {
            if (n != 0) {
                break;
            }
            int n2 = (int)this.getRoleCardinality(s, s2) - 1;
            RoleInfo roleInfo;
            try {
                roleInfo = this.getRoleInfo(relationTypeName, s2);
            }
            catch (RelationTypeNotFoundException ex) {
                throw new RuntimeException(ex.getMessage());
            }
            catch (RoleInfoNotFoundException ex2) {
                throw new RuntimeException(ex2.getMessage());
            }
            if (!roleInfo.checkMinDegree(n2)) {
                n = 1;
            }
            else {
                continue;
            }
        }
        if (n != 0) {
            this.removeRelation(s);
        }
        else {
            for (String s3 : list) {
                if ((relation instanceof RelationSupport)) {
                    try {
                        ((RelationSupport)relation).handleMBeanUnregistrationInt(objectName, s3, true, this);
                    }
                    catch (RelationTypeNotFoundException ex6) {
                        throw new RuntimeException(ex6.getMessage());
                    }
                    catch (InvalidRoleValueException ex7) {
                        throw new RuntimeException(ex7.getMessage());
                    }
                }
                else {
                    Object[] array = { objectName, s3 };
                    String[] array2 = { "javax.management.ObjectName", "java.lang.String" };
                    try {
                        this.myMBeanServer.invoke((ObjectName)relation, "handleMBeanUnregistration", array, array2);
                    }
                    catch (InstanceNotFoundException ex3) {
                        throw new RuntimeException(ex3.getMessage());
                    }
                    catch (ReflectionException ex4) {
                        throw new RuntimeException(ex4.getMessage());
                    }
                    catch (MBeanException ex5) {
                        throw new RuntimeException(ex5.getTargetException().getMessage());
                    }
                }
            }
        }
        JmxProperties.RELATION_LOGGER.exiting(RelationService.class.getName(), "handleReferenceUnregistration");
        return;
    }
}
